@@ -4,8 +4,8 @@
 /* ************************************************************************** */
 /* *                                                                        * */
 /* * project   : libmng                                                     * */
-/* * file      : libmng_callback_xs.c      copyright (c) 2000 G.Juyn        * */
-/* * version   : 1.0.0                                                      * */
+/* * file      : libmng_callback_xs.c      copyright (c) 2000-2002 G.Juyn   * */
+/* * version   : 1.0.2                                                      * */
 /* *                                                                        * */
 /* * purpose   : callback get/set interface (implementation)                * */
 /* *                                                                        * */
@@ -37,6 +37,12 @@
 /* *             0.9.3 - 10/17/2000 - G.Juyn                                * */
 /* *             - added callback to process non-critical unknown chunks    * */
 /* *                                                                        * */
+/* *             1.0.1 - 02/08/2001 - G.Juyn                                * */
+/* *             - added MEND processing callback                           * */
+/* *                                                                        * */
+/* *             1.0.2 - 06/23/2001 - G.Juyn                                * */
+/* *             - added processterm callback                               * */
+/* *                                                                        * */
 /* ************************************************************************** */
 
 #include "libmng.h"
@@ -328,6 +334,27 @@ mng_retcode MNG_DECL mng_setcb_processneed (mng_handle      hHandle,
 
 /* ************************************************************************** */
 
+#ifdef MNG_SUPPORT_READ
+mng_retcode MNG_DECL mng_setcb_processmend (mng_handle      hHandle,
+                                            mng_processmend fProc)
+{
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACE (((mng_datap)hHandle), MNG_FN_SETCB_PROCESSMEND, MNG_LC_START)
+#endif
+
+  MNG_VALIDHANDLE (hHandle)
+  ((mng_datap)hHandle)->fProcessmend = fProc;
+
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACE (((mng_datap)hHandle), MNG_FN_SETCB_PROCESSMEND, MNG_LC_END)
+#endif
+
+  return MNG_NOERROR;
+}
+#endif /* MNG_SUPPORT_READ */
+
+/* ************************************************************************** */
+
 #ifdef MNG_SUPPORT_READ
 mng_retcode MNG_DECL mng_setcb_processunknown (mng_handle         hHandle,
                                                mng_processunknown fProc)
@@ -349,6 +376,27 @@ mng_retcode MNG_DECL mng_setcb_processunknown (mng_handle         hHandle,
 
 /* ************************************************************************** */
 
+#ifdef MNG_SUPPORT_READ
+mng_retcode MNG_DECL mng_setcb_processterm (mng_handle      hHandle,
+                                            mng_processterm fProc)
+{
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACE (((mng_datap)hHandle), MNG_FN_SETCB_PROCESSTERM, MNG_LC_START)
+#endif
+
+  MNG_VALIDHANDLE (hHandle)
+  ((mng_datap)hHandle)->fProcessterm = fProc;
+
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACE (((mng_datap)hHandle), MNG_FN_SETCB_PROCESSTERM, MNG_LC_END)
+#endif
+
+  return MNG_NOERROR;
+}
+#endif /* MNG_SUPPORT_READ */
+
+/* ************************************************************************** */
+
 #ifdef MNG_SUPPORT_DISPLAY
 mng_retcode MNG_DECL mng_setcb_getcanvasline (mng_handle        hHandle,
                                               mng_getcanvasline fProc)
@@ -829,6 +877,25 @@ mng_processneed MNG_DECL mng_getcb_processneed (mng_handle hHandle)
 
 /* ************************************************************************** */
 
+#ifdef MNG_SUPPORT_READ
+mng_processmend MNG_DECL mng_getcb_processmend (mng_handle hHandle)
+{
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACEX (((mng_datap)hHandle), MNG_FN_GETCB_PROCESSMEND, MNG_LC_START)
+#endif
+
+  MNG_VALIDHANDLEX (hHandle)
+
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACEX (((mng_datap)hHandle), MNG_FN_GETCB_PROCESSMEND, MNG_LC_END)
+#endif
+
+  return ((mng_datap)hHandle)->fProcessmend;
+}
+#endif /* MNG_SUPPORT_READ */
+
+/* ************************************************************************** */
+
 #ifdef MNG_SUPPORT_READ
 mng_processunknown MNG_DECL mng_getcb_processunknown (mng_handle hHandle)
 {
@@ -848,6 +915,25 @@ mng_processunknown MNG_DECL mng_getcb_processunknown (mng_handle hHandle)
 
 /* ************************************************************************** */
 
+#ifdef MNG_SUPPORT_READ
+mng_processterm MNG_DECL mng_getcb_processterm (mng_handle hHandle)
+{
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACEX (((mng_datap)hHandle), MNG_FN_GETCB_PROCESSTERM, MNG_LC_START)
+#endif
+
+  MNG_VALIDHANDLEX (hHandle)
+
+#ifdef MNG_SUPPORT_TRACE
+  MNG_TRACEX (((mng_datap)hHandle), MNG_FN_GETCB_PROCESSTERM, MNG_LC_END)
+#endif
+
+  return ((mng_datap)hHandle)->fProcessterm;
+}
+#endif /* MNG_SUPPORT_READ */
+
+/* ************************************************************************** */
+
 #ifdef MNG_SUPPORT_DISPLAY
 mng_getcanvasline MNG_DECL mng_getcb_getcanvasline (mng_handle hHandle)
 {
@@ -1059,3 +1145,5 @@ mng_processarow MNG_DECL mng_getcb_processarow (mng_handle hHandle)
 /* * end of file                                                            * */
 /* ************************************************************************** */
 
+
+
