@@ -4,8 +4,8 @@
 /* ************************************************************************** */
 /* *                                                                        * */
 /* * project   : libmng                                                     * */
-/* * file      : libmng_jpeg.c             copyright (c) 2000 G.Juyn        * */
-/* * version   : 1.0.0                                                      * */
+/* * file      : libmng_jpeg.c             copyright (c) 2000-2002 G.Juyn   * */
+/* * version   : 1.0.5                                                      * */
 /* *                                                                        * */
 /* * purpose   : JPEG library interface (implementation)                    * */
 /* *                                                                        * */
@@ -34,6 +34,18 @@
 /* *             0.9.3 - 10/16/2000 - G.Juyn                                * */
 /* *             - added support for JDAA                                   * */
 /* *                                                                        * */
+/* *             1.0.1 - 04/19/2001 - G.Juyn                                * */
+/* *             - added export of JPEG functions for DLL                   * */
+/* *             1.0.1 - 04/22/2001 - G.Juyn                                * */
+/* *             - fixed memory-leaks (Thanks Gregg!)                       * */
+/* *                                                                        * */
+/* *             1.0.4 - 06/22/2002 - G.Juyn                                * */
+/* *             - B526138 - returned IJGSRC6B calling convention to        * */
+/* *               default for MSVC                                         * */
+/* *                                                                        * */
+/* *             1.0.5 - 24/02/2003 - G.Juyn                                * */
+/* *             - B683152 - libjpeg suspension not always honored correctly* */
+/* *                                                                        * */
 /* ************************************************************************** */
 
 #include "libmng.h"
@@ -53,7 +65,7 @@
 
 /* ************************************************************************** */
 
-#ifdef MNG_INCLUDE_JNG
+#if defined(MNG_INCLUDE_JNG) && defined(MNG_INCLUDE_DISPLAY_PROCS)
 
 /* ************************************************************************** */
 /* *                                                                        * */
@@ -66,7 +78,11 @@
 /* ************************************************************************** */
 
 #ifdef MNG_INCLUDE_JNG_READ
+#ifdef MNG_DEFINE_JPEG_STDCALL
+void MNG_DECL mng_init_source (j_decompress_ptr cinfo)
+#else
 void mng_init_source (j_decompress_ptr cinfo)
+#endif
 {
   return;                              /* nothing needed */
 }
@@ -75,7 +91,11 @@ void mng_init_source (j_decompress_ptr cinfo)
 /* ************************************************************************** */
 
 #ifdef MNG_INCLUDE_JNG_READ
+#ifdef MNG_DEFINE_JPEG_STDCALL
+boolean MNG_DECL mng_fill_input_buffer (j_decompress_ptr cinfo)
+#else
 boolean mng_fill_input_buffer (j_decompress_ptr cinfo)
+#endif
 {
   return FALSE;                        /* force IJG routine to return to caller */
 }
@@ -84,7 +104,11 @@ boolean mng_fill_input_buffer (j_decompress_ptr cinfo)
 /* ************************************************************************** */
 
 #ifdef MNG_INCLUDE_JNG_READ
+#ifdef MNG_DEFINE_JPEG_STDCALL
+void MNG_DECL mng_skip_input_data (j_decompress_ptr cinfo, long num_bytes)
+#else
 void mng_skip_input_data (j_decompress_ptr cinfo, long num_bytes)
+#endif
 {
   if (num_bytes > 0)                   /* ignore fony calls */
   {                                    /* address my generic structure */
@@ -113,7 +137,11 @@ void mng_skip_input_data (j_decompress_ptr cinfo, long num_bytes)
 /* ************************************************************************** */
 
 #ifdef MNG_INCLUDE_JNG_READ
+#ifdef MNG_DEFINE_JPEG_STDCALL
+void MNG_DECL mng_skip_input_data2 (j_decompress_ptr cinfo, long num_bytes)
+#else
 void mng_skip_input_data2 (j_decompress_ptr cinfo, long num_bytes)
+#endif
 {
   if (num_bytes > 0)                   /* ignore fony calls */
   {                                    /* address my generic structure */
@@ -142,7 +170,11 @@ void mng_skip_input_data2 (j_decompress_ptr cinfo, long num_bytes)
 /* ************************************************************************** */
 
 #ifdef MNG_INCLUDE_JNG_READ
+#ifdef MNG_DEFINE_JPEG_STDCALL
+void MNG_DECL mng_term_source (j_decompress_ptr cinfo)
+#else
 void mng_term_source (j_decompress_ptr cinfo)
+#endif
 {
   return;                              /* nothing needed */
 }
@@ -151,7 +183,11 @@ void mng_term_source (j_decompress_ptr cinfo)
 /* ************************************************************************** */
 
 #ifdef MNG_USE_SETJMP
+#ifdef MNG_DEFINE_JPEG_STDCALL
+void MNG_DECL mng_error_exit (j_common_ptr cinfo)
+#else
 void mng_error_exit (j_common_ptr cinfo)
+#endif
 {                                      /* address my generic structure */
   mng_datap pData = (mng_datap)cinfo->client_data;
 
@@ -166,7 +202,11 @@ void mng_error_exit (j_common_ptr cinfo)
 /* ************************************************************************** */
 
 #ifdef MNG_USE_SETJMP
+#ifdef MNG_DEFINE_JPEG_STDCALL
+void MNG_DECL mng_output_message (j_common_ptr cinfo)
+#else
 void mng_output_message (j_common_ptr cinfo)
+#endif
 {
   return;                              /* just do nothing ! */
 }
@@ -216,12 +256,18 @@ mng_retcode mngjpeg_initialize (mng_datap pData)
                                        /* enable reverse addressing */
   pData->pJPEGcinfo->client_data = pData;
 #endif
-                                       /* initialize temporary buffers */
-  pData->iJPEGbufmax       = MNG_JPEG_MAXBUF;
-  MNG_ALLOC (pData, pData->pJPEGbuf, pData->iJPEGbufmax)
 
-  pData->iJPEGbufmax2      = MNG_JPEG_MAXBUF;
-  MNG_ALLOC (pData, pData->pJPEGbuf2, pData->iJPEGbufmax2)
+  if (pData->pJPEGbuf   == MNG_NULL)   /* initialize temporary buffers */
+  {
+    pData->iJPEGbufmax     = MNG_JPEG_MAXBUF;
+    MNG_ALLOC (pData, pData->pJPEGbuf, pData->iJPEGbufmax)
+  }
+
+  if (pData->pJPEGbuf2  == MNG_NULL) 
+  {
+    pData->iJPEGbufmax2    = MNG_JPEG_MAXBUF;
+    MNG_ALLOC (pData, pData->pJPEGbuf2, pData->iJPEGbufmax2)
+  }
 
   pData->pJPEGcurrent      = pData->pJPEGbuf;
   pData->iJPEGbufremain    = 0;
@@ -241,6 +287,7 @@ mng_retcode mngjpeg_initialize (mng_datap pData)
   pData->bJPEGhasheader    = MNG_FALSE;
   pData->bJPEGdecostarted  = MNG_FALSE;
   pData->bJPEGscanstarted  = MNG_FALSE;
+  pData->bJPEGscanending   = MNG_FALSE;
 
   pData->bJPEGdecompress2  = MNG_FALSE;
   pData->bJPEGhasheader2   = MNG_FALSE;
@@ -292,9 +339,14 @@ mng_retcode mngjpeg_cleanup (mng_datap pData)
 
 #endif /* MNG_INCLUDE_IJG6B */
                                        /* cleanup temporary buffers */
-  MNG_FREE (pData, pData->pJPEGbuf,  pData->iJPEGbufmax)
   MNG_FREE (pData, pData->pJPEGbuf2, pData->iJPEGbufmax2)
+  MNG_FREE (pData, pData->pJPEGbuf,  pData->iJPEGbufmax)
                                        /* cleanup space for JPEG structures */
+#ifdef MNG_INCLUDE_JNG_WRITE
+  MNG_FREE (pData, pData->pJPEGcinfo,  sizeof (mngjpeg_comp  ))
+  MNG_FREE (pData, pData->pJPEGcerr,   sizeof (mngjpeg_error ))
+#endif
+
 #ifdef MNG_INCLUDE_JNG_READ
   MNG_FREE (pData, pData->pJPEGdinfo,  sizeof (mngjpeg_decomp))
   MNG_FREE (pData, pData->pJPEGdsrc,   sizeof (mngjpeg_source))
@@ -304,13 +356,8 @@ mng_retcode mngjpeg_cleanup (mng_datap pData)
   MNG_FREE (pData, pData->pJPEGderr2,  sizeof (mngjpeg_error ))
 #endif
 
-#ifdef MNG_INCLUDE_JNG_WRITE
-  MNG_FREE (pData, pData->pJPEGcinfo,  sizeof (mngjpeg_comp  ))
-  MNG_FREE (pData, pData->pJPEGcerr,   sizeof (mngjpeg_error ))
-#endif
-
-  MNG_FREE (pData, pData->pJPEGrow,  pData->iJPEGrowlen)
   MNG_FREE (pData, pData->pJPEGrow2, pData->iJPEGrowlen2)
+  MNG_FREE (pData, pData->pJPEGrow,  pData->iJPEGrowlen)
                                        /* whatever we were doing ... */
                                        /* we don't anymore ... */
   pData->bJPEGcompress     = MNG_FALSE;
@@ -319,6 +366,7 @@ mng_retcode mngjpeg_cleanup (mng_datap pData)
   pData->bJPEGhasheader    = MNG_FALSE;
   pData->bJPEGdecostarted  = MNG_FALSE;
   pData->bJPEGscanstarted  = MNG_FALSE;
+  pData->bJPEGscanending   = MNG_FALSE;
 
   pData->bJPEGdecompress2  = MNG_FALSE;
   pData->bJPEGhasheader2   = MNG_FALSE;
@@ -537,7 +585,8 @@ mng_retcode mngjpeg_decompressdata (mng_datap  pData,
                                        /* process some scanlines ? */
     if ((pData->bJPEGhasheader) && (pData->bJPEGdecostarted) &&
 	    ((!jpeg_input_complete (pData->pJPEGdinfo)) ||
-         (pData->pJPEGdinfo->output_scanline < pData->pJPEGdinfo->output_height)))
+         (pData->pJPEGdinfo->output_scanline < pData->pJPEGdinfo->output_height) ||
+         ((pData->bJPEGprogressive) && (pData->bJPEGscanending))))
     {
       mng_int32 iLines;
 
@@ -547,9 +596,7 @@ mng_retcode mngjpeg_decompressdata (mng_datap  pData,
         JSAMPROW pRow = (JSAMPROW)pData->pJPEGrow;
 
                                        /* init new pass ? */
-        if ((pData->bJPEGprogressive) &&
-            ((!pData->bJPEGscanstarted) ||
-             (pData->pJPEGdinfo->output_scanline >= pData->pJPEGdinfo->output_height)))
+        if ((pData->bJPEGprogressive) && (!pData->bJPEGscanstarted))
         {
           pData->bJPEGscanstarted = MNG_TRUE;
 
@@ -566,31 +613,34 @@ mng_retcode mngjpeg_decompressdata (mng_datap  pData,
         }
 
         /* while (scan lines remain to be read) */
-        do
+        if ((!pData->bJPEGprogressive) || (!pData->bJPEGscanending))
         {
+          do
+          {
           /*   jpeg_read_scanlines(...); */
 #ifdef MNG_SUPPORT_TRACE
-          MNG_TRACE (pData, MNG_FN_JPEG_DECOMPRESSDATA, MNG_LC_JPEG_READ_SCANLINES)
+            MNG_TRACE (pData, MNG_FN_JPEG_DECOMPRESSDATA, MNG_LC_JPEG_READ_SCANLINES)
 #endif
-          iLines = jpeg_read_scanlines (pData->pJPEGdinfo, (JSAMPARRAY)&pRow, 1);
+            iLines = jpeg_read_scanlines (pData->pJPEGdinfo, (JSAMPARRAY)&pRow, 1);
 
-          pData->pJPEGcurrent   = (mng_uint8p)pData->pJPEGdinfo->src->next_input_byte;
-          pData->iJPEGbufremain = (mng_uint32)pData->pJPEGdinfo->src->bytes_in_buffer;
+            pData->pJPEGcurrent   = (mng_uint8p)pData->pJPEGdinfo->src->next_input_byte;
+            pData->iJPEGbufremain = (mng_uint32)pData->pJPEGdinfo->src->bytes_in_buffer;
 
-          if (iLines > 0)              /* got something ? */
-          {
-            if (pData->fStorerow2)     /* store in object ? */
+            if (iLines > 0)            /* got something ? */
             {
-              iRetcode = ((mng_storerow)pData->fStorerow2) (pData);
+              if (pData->fStorerow2)   /* store in object ? */
+              {
+                iRetcode = ((mng_storerow)pData->fStorerow2) (pData);
 
-              if (iRetcode)            /* on error bail out */
+                if (iRetcode)          /* on error bail out */
                 return iRetcode;
 
+              }
             }
           }
+          while ((pData->pJPEGdinfo->output_scanline < pData->pJPEGdinfo->output_height) &&
+                 (iLines > 0));        /* until end-of-image or not enough input-data */
         }
-        while ((pData->pJPEGdinfo->output_scanline < pData->pJPEGdinfo->output_height) &&
-               (iLines > 0));          /* until end-of-image or not enough input-data */
 
         /* terminate output pass */
         if ((pData->bJPEGprogressive) &&
@@ -599,16 +649,23 @@ mng_retcode mngjpeg_decompressdata (mng_datap  pData,
 #ifdef MNG_SUPPORT_TRACE
           MNG_TRACE (pData, MNG_FN_JPEG_DECOMPRESSDATA, MNG_LC_JPEG_FINISH_OUTPUT)
 #endif
-          jpeg_finish_output (pData->pJPEGdinfo);
-                                       /* this scan has ended */
-          pData->bJPEGscanstarted = MNG_FALSE;
+          if (jpeg_finish_output (pData->pJPEGdinfo) != JPEG_SUSPENDED)
+          {                            /* this scan has ended */
+            pData->bJPEGscanstarted = MNG_FALSE;
+            pData->bJPEGscanending  = MNG_FALSE;
+          }
+          else
+          {
+            pData->bJPEGscanending  = MNG_TRUE;
+          }
         }
       }
-      while ((!jpeg_input_complete (pData->pJPEGdinfo)) && (iLines > 0));
+      while ((!jpeg_input_complete (pData->pJPEGdinfo)) &&
+             (iLines > 0) && (!pData->bJPEGscanending));
     }
                                        /* end of image ? */
     if ((pData->bJPEGhasheader) && (pData->bJPEGdecostarted) &&
-        (jpeg_input_complete (pData->pJPEGdinfo)) &&
+        (!pData->bJPEGscanending) && (jpeg_input_complete (pData->pJPEGdinfo)) &&
         (pData->pJPEGdinfo->input_scan_number == pData->pJPEGdinfo->output_scan_number))
     {
       /* jpeg_finish_decompress(...); */
@@ -656,6 +713,8 @@ mng_retcode mngjpeg_decompressfree (mng_datap pData)
   if (iRetcode != 0)                   /* got here from longjmp ? */
     MNG_ERRORJ (pData, iRetcode)       /* then IJG-lib issued an error */
 #endif
+                                       /* free the row of JPEG-samples*/
+  MNG_FREE (pData, pData->pJPEGrow, pData->iJPEGrowlen)
 
   /* release the JPEG decompression object */
 #ifdef MNG_SUPPORT_TRACE
@@ -933,7 +992,8 @@ mng_retcode mngjpeg_decompressdata2 (mng_datap  pData,
 #ifdef MNG_SUPPORT_TRACE
           MNG_TRACE (pData, MNG_FN_JPEG_DECOMPRESSDATA, MNG_LC_JPEG_FINISH_OUTPUT)
 #endif
-          jpeg_finish_output (pData->pJPEGdinfo2);
+          if (jpeg_finish_output (pData->pJPEGdinfo2) == JPEG_SUSPENDED)
+            jpeg_finish_output (pData->pJPEGdinfo2);
                                        /* this scan has ended */
           pData->bJPEGscanstarted2 = MNG_FALSE;
         }
@@ -990,6 +1050,8 @@ mng_retcode mngjpeg_decompressfree2 (mng_datap pData)
   if (iRetcode != 0)                   /* got here from longjmp ? */
     MNG_ERRORJ (pData, iRetcode)       /* then IJG-lib issued an error */
 #endif
+                                       /* free the row of JPEG-samples*/
+  MNG_FREE (pData, pData->pJPEGrow2, pData->iJPEGrowlen2)
 
   /* release the JPEG decompression object */
 #ifdef MNG_SUPPORT_TRACE
@@ -1011,7 +1073,7 @@ mng_retcode mngjpeg_decompressfree2 (mng_datap pData)
 
 /* ************************************************************************** */
 
-#endif /* MNG_INCLUDE_JNG */
+#endif /* MNG_INCLUDE_JNG && MNG_INCLUDE_DISPLAY_PROCS */
 
 /* ************************************************************************** */
 /* * end of file                                                            * */
